@@ -2,9 +2,10 @@ package FFI::Platypus::Lang::CPP;
 
 use strict;
 use warnings;
+use FFI::ExtractSymbols qw( extract_symbols );
 use FFI::Platypus;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 NAME
 
@@ -296,6 +297,15 @@ Returns a subroutine reference that will "mangle" C++ names.
 
 =cut
 
+if(eval { require FFI::Platypus::Lang::CPP::Demangle::XS })
+{
+  *_demangle = \&FFI::Platypus::Lang::CPP::Demangle::XS::demangle;
+}
+else
+{
+  *_demangle = sub { `c++filt $_[0]` };
+}
+
 sub mangler
 {
   my($class, @libs) = @_;
@@ -304,19 +314,15 @@ sub mangler
   
   foreach my $libpath (@libs)
   {
-    require Parse::nm;
-    Parse::nm->run(
-      files => $libpath,
-      filters => [ {
-        action => sub {
-          my $c_symbol = $_[0];
-          # TODO: what to do if we do not have c++filt?
-          my $cpp_symbol = `c++filt $c_symbol`;
-          chomp $cpp_symbol;
-          return if $c_symbol eq $cpp_symbol;
-          $mangle{$cpp_symbol} = $c_symbol;
-        },
-      } ],
+    extract_symbols($libpath,
+      export => sub {
+        my($symbol1, $symbol2) = @_;
+        my $cpp_symbol = _demangle($symbol2);
+        return unless defined $cpp_symbol;
+        chomp $cpp_symbol;
+        return if $cpp_symbol eq $symbol2;
+        $mangle{$cpp_symbol} = $symbol1;
+      },
     );
   }
   
